package weblogic.deploy.service.internal.transport;

import java.io.*;
import java.util.*;
import weblogic.deploy.service.internal.DomainVersion;
import weblogic.common.internal.WLObjectInputStream;
import weblogic.common.internal.WLObjectOutputStream;

public final class DeploymentServiceMessage implements Externalizable {

    public DeploymentServiceMessage() {
      timeoutValue=-1L;
      targets=null;
      items=null;
      fromVersion=null;
      toVersion=null;
      deploymentType=null;
      needsVersionUpdate = true;
      currentDeploymentServiceVersion=2;
      deploymentServiceVersion=2;
    }

    public final void writeExternal(ObjectOutput oo) throws IOException {
      oo.writeByte(deploymentServiceVersion);
      writeMessage(deploymentServiceVersion, oo);
    }

    private void writeMessage(byte deploymentServiceVersion,ObjectOutput oo) throws IOException {
      oo.writeByte(messageType);
      oo.writeLong(deploymentId);
      oo.writeLong(timeoutValue);
      oo.writeBoolean(callConfigurationProviderLast);
      ((WLObjectOutputStream)oo).writeString(messageSrc);
      boolean itemsIncluded=(items!=null);
      oo.writeBoolean(itemsIncluded);
      if (itemsIncluded) ((WLObjectOutputStream)oo).writeArrayList(items);
      boolean fromVersionIncluded=(fromVersion!=null);
      oo.writeBoolean(fromVersionIncluded);
      if (fromVersionIncluded) oo.writeObject(fromVersion);
      boolean toVersionIncluded=(toVersion!=null);
      oo.writeBoolean(toVersionIncluded);
      if (toVersionIncluded) oo.writeObject(toVersion);
      boolean initiatorIncluded=false;
      oo.writeBoolean(initiatorIncluded);
      boolean deploymentTypeIncluded=(deploymentType!=null);
      oo.writeBoolean(deploymentTypeIncluded);
      if (deploymentTypeIncluded) oo.writeObject(deploymentType);
      oo.writeBoolean(needsVersionUpdate);
    }

    public final void readExternal(ObjectInput oi) throws IOException, ClassNotFoundException {
      deploymentServiceVersion=oi.readByte();
      readMessage(deploymentServiceVersion,oi);
    }

    private void readMessage(byte deploymentServiceVersion, ObjectInput oi) throws IOException, ClassNotFoundException {
      messageType=oi.readByte();
      deploymentId=oi.readLong();
      timeoutValue=oi.readLong();
      callConfigurationProviderLast=oi.readBoolean();
      messageSrc=((WLObjectInputStream)oi).readString();
      boolean itemsIncluded =oi.readBoolean();
      if (itemsIncluded) items=((WLObjectInputStream)oi).readArrayList();
      boolean fromVersionIncluded=oi.readBoolean();
      if (fromVersionIncluded) fromVersion=(DomainVersion)oi.readObject();
      boolean toVersionIncluded=oi.readBoolean();
      if (toVersionIncluded) toVersion=(DomainVersion)oi.readObject();
      boolean initiatorIncluded=oi.readBoolean();
      if (initiatorIncluded) {
        System.out.println("ERROR: EXPECTED INITIATOR");
      }
      boolean deploymentTypeIncluded=oi.readBoolean();
      if (deploymentTypeIncluded) setDeploymentType((String)oi.readObject());
      needsVersionUpdate=oi.readBoolean();
    }

    public final String getDeploymentType() {
      return deploymentType;
    }

    public final void setDeploymentType(String given) {
      if (messageType!=13) {
         throw new IllegalArgumentException("setDeploymentType is invalid for this message type");
      } else {
         deploymentType = given;
         return;
      }
    }

    private static final long serialVersionUID = 0xf82423759a451c47L;

    public static final byte HEARTBEAT = 0;
    public static final byte REQUEST_PREPARE = 1;
    public static final byte REQUEST_COMMIT = 2;
    public static final byte REQUEST_CANCEL = 3;
    public static final byte GET_DEPLOYMENTS = 4;
    public static final byte GET_DEPLOYMENTS_RESPONSE = 5;
    public static final byte PREPARE_ACK = 6;
    public static final byte PREPARE_NAK = 7;
    public static final byte COMMIT_SUCCEEDED = 8;
    public static final byte COMMIT_FAILED = 9;
    public static final byte CANCEL_SUCCEEDED = 10;
    public static final byte CANCEL_FAILED = 11;
    public static final byte REQUEST_STATUS = 12;
    public static final byte BLOCKING_GET_DEPLOYMENTS = 13;

    public byte deploymentServiceVersion;
    public final byte currentDeploymentServiceVersion;
    public byte messageType;
    public long deploymentId;
    public long timeoutValue;
    public boolean callConfigurationProviderLast;
    public String messageSrc;
    public transient Set targets;
    public ArrayList items;
    public DomainVersion fromVersion;
    public DomainVersion toVersion;
    public String deploymentType;
    public boolean needsVersionUpdate;
}
