package weblogic.common.internal;

import java.io.*;
import java.util.*;
import weblogic.utils.io.DelegatingInputStream;

public class WLObjectInputStream extends ObjectInputStream {

    public WLObjectInputStream(InputStream is) throws IOException {
      this(new DelegatingInputStream(is));
    }

    private WLObjectInputStream(DelegatingInputStream delegate) throws IOException {
      super(delegate);

      try {
        enableResolveObject(true);
      } catch(SecurityException ex) { }

      _flddelegate = delegate;
    }

    public final void setDelegate(InputStream in) {
      _flddelegate.setDelegate(in);
    }

    protected final Class resolveClass(ObjectStreamClass v) throws IOException, ClassNotFoundException {
      String annotation = null;
      try {
        annotation = readUTF();
      } catch(OptionalDataException ex) { }
      
      return Thread.currentThread().getContextClassLoader().loadClass(annotation);
    }

    protected boolean useInterAppClassLoader() {
      return false;
    }

    protected Class resolveProxyClass(String interfaces[]) throws IOException, ClassNotFoundException {
      String annotation = null;
      
      try {
        annotation = readUTF();
      } catch(OptionalDataException ex) { }

      return Thread.currentThread().getContextClassLoader().loadClass(annotation);
    }

    public String readUTF() throws IOException {
      return readUTF();
    }

    protected Object resolveObject(Object obj) throws IOException {
      return obj;
    }

    protected final void readStreamHeader() throws IOException {
    }

    public final void reset() throws IOException {
    }

    public final void close() throws IOException {
    }

    public Object readObjectWL() throws IOException, ClassNotFoundException {
      return readObject();
    }

    public final String readString() throws IOException {
      byte b=readByte();
       
      if (b==112) return null;
        else return readUTF();
    }

    public final Date readDate() throws IOException, ClassNotFoundException {
      return (Date)readObject();
    }

    public final ArrayList readArrayList() throws IOException, ClassNotFoundException {
      return (ArrayList)readObject();
    }

    public final Properties readProperties() throws IOException, ClassNotFoundException {
      return (Properties)readObject();
    }

    public final byte[] readBytes() throws IOException, ClassNotFoundException {
      return (byte[])(byte[])readObject();
    }

    public final Object[] readArrayOfObjects() throws IOException, ClassNotFoundException {
      return (Object[])(Object[])readObject();
    }

    public Object readImmutable() throws IOException, ClassNotFoundException {
      return readObject();
    }

    public final String readAbbrevString() throws IOException {
      return readString();
    }

    private final DelegatingInputStream _flddelegate;
}
