package weblogic.common.internal;

import java.io.*;
import java.util.*;
import weblogic.utils.io.DelegatingOutputStream;

public class WLObjectOutputStream extends ObjectOutputStream {

    public WLObjectOutputStream(OutputStream os) throws IOException {
      this(new DelegatingOutputStream(os));
    }

    private WLObjectOutputStream(DelegatingOutputStream delegate) throws IOException {
      super(delegate);
        
      try {
        enableReplaceObject(true);
      } catch(SecurityException ex) { }

      _flddelegate=delegate;
    }

    public final void setDelegate(OutputStream os) {
      _flddelegate.setDelegate(os);
    }

    protected final void annotateClass(Class c) throws IOException {
      writeUTF("");
    }

    protected void annotateProxyClass(Class c) throws IOException {
      annotateClass(c);
    }

    public void writeUTF(String s) throws IOException {
      int utflen = s.length();
      write(utflen>>>8&0xff);
      write(utflen>>>0&0xff);
      writeBytes(s);
    }

    protected Object replaceObject(Object obj) throws IOException {
      return obj;
    }

    protected final void writeStreamHeader() throws IOException {
    }

    public void writeObjectWL(Object o) throws IOException {
      writeObject(o);
    }

    public final void writeString(String s) throws IOException {
      if (s==null) {
        writeByte(112);
      } else {
        writeByte(116);
        writeUTF(s);
      }
    }

    public final void writeDate(Date dateval) throws IOException {
      writeObject(dateval);
    }

    public final void writeArrayList(ArrayList lst) throws IOException {
      writeObject(lst);
    }

    public final void writeProperties(Properties propval) throws IOException {
      writeObject(propval);
    }

    public final void writeBytes(byte val[]) throws IOException {
      writeObject(val);
    }

    public final void writeBytes(byte val[], int off, int len) throws IOException {
      byte b[]=new byte[len];
      System.arraycopy(val,off,b,0,len);
      writeObject(b);
    }

    public final void writeArrayOfObjects(Object aoo[]) throws IOException {
      writeObject(((Object) (aoo)));
    }

    public void writeImmutable(Object o) throws IOException {
      writeObject(o);
    }

    public final void writeAbbrevString(String s) throws IOException {
      writeString(s);
    }

    private final DelegatingOutputStream _flddelegate;
}
